import java.awt.Point;
import java.util.Arrays;

/**
 * By Steve Putz, 18-March-2014
 * Adapted from a program by Ethan Arns and examples from http://rosettacode.org/wiki/Knight's_tour
 */
public class WarnsdorffKnight
{
    static final private int ROWS = ChessBoard.ROWS;
    static final private int COLS = ChessBoard.COLS;
    static final private int[] colOff = { 1,  2,  2,  1, -1, -2, -2, -1};
    static final private int[] rowOff = {-2, -1,  1,  2,  2,  1, -1, -2};
    private int[][] board;
    private int iterations = 0;


    public static Point[] findSolution(int x, int y)
    {
        WarnsdorffKnight kn = new WarnsdorffKnight();
        if (kn.placeKnight(y, x, 0)) {
//             kn.print();
        } else {
            System.out.println("No solution found");
        }
        return kn.getSolution();
    }

    public Point[] getSolution() {
        Point[] steps = new Point[ROWS*COLS];
        for (int row = 0; row < ROWS; row++) {
            for (int col = 0; col < COLS; col++) {
                int i = board[row][col];
                if (i > 0) {
                    steps[i-1] = new Point(col, row);
                }
            }
        }
        return steps;
    }

    public WarnsdorffKnight()
    {
        board = new int[ROWS][COLS];    //Make table
        // no need to fill with zeros since that is the Java default
//         for(int row = 0; row < ROWS; row++) {
//             for(int col = 0; col < COLS; col++) {
//                 board[row][col] = 0;    //Fill empty
//             }
//         }
    }

    public void print()
    {
        System.out.print("  ");//upper heading
        for(int i = 1; i <= COLS; i++) {
            System.out.print(String.format("%2d ", i));
        }
        System.out.println();
        for (int row = 0; row < ROWS; row++) {
            System.out.print(String.format("%2d ", (row+1)));
            for (int col = 0; col < COLS; col++) {
                System.out.print(String.format("%02d ", board[row][col]));
            }
            System.out.println();
        }
        System.out.println("Solved in " + iterations + " recursions!");
    }

    /**
     * return the number of possible next moves from row, col
     */ 
    private int countOpen(int row, int col)
    {
        int open = 0;
        for(int i = 0; i < 8; i++) {
            int r = row+rowOff[i];
            int c = col+colOff[i];
            if (0 <= r && r < ROWS && 0 <= c && c < COLS && board[r][c] == 0) open++;
        }
        return open;
    }
    
    public boolean placeKnight(int row, int col, int attempt)
    {
        if (0 <= row && row < ROWS && 0 <= col && col < COLS && board[row][col] == 0)
        {
            iterations++;
            int moveCount = attempt+1;
            board[row][col] = moveCount;
            if (moveCount >= ROWS*COLS) {
                return true;
            }
            int[] order = new int[8];
            for(int i = 0; i < 8; i++) {
                int r = row+rowOff[i];
                int c = col+colOff[i];
                if (0 <= r && r < ROWS && 0 <= c && c < COLS) {
                    int priority = 10 * countOpen(r, c) + i;
                    order[i] = priority;
                }
            }
            Arrays.sort(order);
            for(int i = 0; i < 8; i++) {
                if (order[i] == 0) continue;
                int moveIndex = order[i] % 10;   // get the move index
                if (placeKnight(row+rowOff[moveIndex], col+colOff[moveIndex], moveCount)) return true;
            }
            board[row][col] = 0;
        }
        return false;
    }

}